#!/usr/bin/env python

#Import Modules
import random
import string
import sys
import os, os.path
import pygame
from pygame.locals import *



#functions to create our resources
def load_image(name, colorkey=None):
    fullname = os.path.join('data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()
    fullname = os.path.join('data', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print 'Cannot load sound:', fullname
        #raise SystemExit, message
        return NoneSound()
    return sound

def load_screen():
    global blocked_tile_list
    global mapx
    global mapy

    object_sprites.empty()
    enemy_sprites.empty()
    enemy_weapon_sprites.empty()
    player_weapon_sprites.empty()
    blocked_tile_list = []

    full_name = os.path.join('saved', user_name+str(mapx)+"x"+str(mapy)+".map")
    if os.path.exists(full_name) == False:
        full_name = os.path.join('maps', "m"+str(mapx)+"x"+str(mapy)+".map")

    
    fin = file(full_name,'r')
    x = 0
    y = 0
    for j in range(0,15):
        line = fin.readline()
        cells = string.split(line)
        x=0
        for i in cells:
            elem = string.split(i, ",")
            #elem[0] == image
            #elem[1] == walkable

            tile_image, tile_rect = load_image(elem[0], None)
            tile_rect.topleft = (x,y)
            tile_image.convert()
            background.blit(tile_image, tile_rect)
            x += 40
            #if x == 600:
            #    x = 0
            #    y += 40
            
            if elem[1] == "False":
                #add rect to list
                blocked_tile_list[1:1] = [tile_rect]
    
        y+=40
    
    whole = fin.read()
    lines = string.split(whole, "\n")
    for j in lines:
        elem = string.split(j)
        if elem:
            if elem[0] == "object":
                x, y = string.split(elem[1],',')
                pos = int(x)*40-20, int(y)*40-20
                #print "Object Found at", pos, "Image is", elem[2], "code is", elem[3]
                #build the command
                com = ""
                for k in range(3, len(elem)):
                    com+=elem[k]+ ' '
                object_sprites.add(Object(elem[2], pos,com))
            elif elem[0] == "enemy":
                #enemy x,y image velocity projectile_image
                x, y = string.split(elem[1],',')
                pos = int(x)*40-20, int(y)*40-20
                #print "Enemy Found at", x, y, "Image is", elem[2]
                if len(elem) == 7:                
                    enemy_sprites.add(Enemy(elem[2], pos, int(elem[3]), int(elem[4]), int(elem[5]), elem[6]))
                else:
                    enemy_sprites.add(Enemy(elem[2], pos, int(elem[3]), int(elem[4]), int(elem[5])))
            elif elem[0] == "boss1":
                enemy_sprites.add(Boss1())
            elif elem[0] == "boss2":
                enemy_sprites.add(Boss2())
                
    
    screen.blit(background, (0,0))
    fin.close()

    print_all_text()    

    return blocked_tile_list


def save_screen():
    global mapx
    global mapy

    full_name = os.path.join('maps', "m"+str(mapx)+"x"+str(mapy)+".map")
    fin = file(full_name,'r')
    whole = fin.read()
    fin.close()

    full_name = os.path.join('saved', user_name+str(mapx)+"x"+str(mapy)+".map")
    fout = file(full_name,'w')
    
    lines = string.split(whole, "\n")
    for j in lines:
        elem = string.split(j)
        if elem[0] == "object":
            pass
        elif elem[0] == "boss1":
            pass
        elif elem[0] == "boss2":
            pass
        else:
            fout.write(j+"\n")


def load_game():
    global user_name
    global player
    global money
    global s_weapon_image, s_weapon_damage, s_weapon_sound_name, s_weapon_sound
    global a_weapon_image, a_weapon_damage, a_weapon_sound_name, a_weapon_sound
    
    full_name = os.path.join('saved', user_name+".sav")
    fin = file(full_name,'r')
    whole = fin.read()
    health_max,s_weapon_image,s_weapon_damage,s_weapon_sound_name,a_weapon_image,a_weapon_damage,a_weapon_sound_name, money, blank = string.split(whole, "\n")
    player.max_health = int(health_max)
    s_weapon_damage = int(s_weapon_damage)
    a_weapon_damage = int(a_weapon_damage)
    money = int(money)
    fin.close()
    s_weapon_sound = load_sound(s_weapon_sound_name)
    a_weapon_sound = load_sound(a_weapon_sound_name)
    
def save_game():
    global user_name
    global player
    global s_weapon_image, s_weapon_damage, s_weapon_sound_name
    global a_weapon_image, a_weapon_damage, a_weapon_sound_name
    full_name = os.path.join('saved', user_name+".sav")
    fout = file(full_name,'w')
    fout.write(str(player.max_health)+"\n")
    fout.write(s_weapon_image+"\n")
    fout.write(str(s_weapon_damage)+"\n")
    fout.write(s_weapon_sound_name+"\n")
    fout.write(a_weapon_image+"\n")
    fout.write(str(a_weapon_damage)+"\n")
    fout.write(a_weapon_sound_name+"\n")
    fout.write(str(money)+"\n")
    fout.close()

def clear_hp_text():
    global player
    #actual HP
    text = font.render(str(player.health), 1, (0, 0, 0))
    screen.blit(text, textpos)
    # /max hp
    label = font.render("/"+str(player.max_health), 1, (0, 0, 0))
    label_pos = label.get_rect()
    label_pos.topleft = (705,20)
    screen.blit(label,label_pos) 

def update_hp_text():
    global player
    #actual HP
    text = font.render(str(player.health), 1, (200, 200, 200))
    screen.blit(text, textpos)
    # /max hp
    label = font.render("/"+str(player.max_health), 1, (200, 200, 200))
    label_pos = label.get_rect()
    label_pos.topleft = (705,20)
    screen.blit(label,label_pos) 

def clear_coins_text():
    # actual coins
    label = font.render(str(money), 1, (0, 0, 0))
    label_pos = label.get_rect()
    label_pos.topleft = (705,50)
    screen.blit(label,label_pos)
        
def update_coins_text():
    # actual coins
    label = font.render(str(money), 1, (200, 200, 200))
    label_pos = label.get_rect()
    label_pos.topleft = (705,50)
    screen.blit(label,label_pos)

def print_all_text():
    # HP:
    label = font.render("HP:", 1, (200, 200, 200))
    label_pos = label.get_rect()
    label_pos.topleft = (630,20)
    screen.blit(label,label_pos)
    #actual HP
    text = font.render(str(player.health), 1, (200, 200, 200))
    screen.blit(text, textpos)
    # /max hp
    label = font.render("/"+str(player.max_health), 1, (200, 200, 200))
    label_pos.topleft = (705,20)
    screen.blit(label,label_pos)

    # coins:
    label = font.render("Coins:", 1, (200, 200, 200))
    label_pos = label.get_rect()
    label_pos.topleft = (630,50)
    screen.blit(label,label_pos)
    # actual coins
    label = font.render(str(money), 1, (200, 200, 200))
    label_pos.topleft = (705,50)
    screen.blit(label,label_pos)

class Stars(pygame.sprite.Sprite):
    def __init__(self, pos):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        self.pos = pos
        self.flag = 0
        global star_images
        self.image = star_images[0]
        self.rect = star_rects[0]
        self.rect.center = self.pos

    def update(self):
        global star_images
        if self .flag == 3:
            self.image = star_images[1]
            self.rect = star_rects[1]
            self.rect.center = self.pos
        elif self.flag == 6:
            self.image = star_images[2]
            self.rect = star_rects[2]
            self.rect.center = self.pos
        self.flag += 1
        if self.flag == 9:
            self.kill()
        

class Player(pygame.sprite.Sprite):
    """This class is for the players character"""
    def __init__(self):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        screen = pygame.display.get_surface()
        self.flag = 1
        self.direction = 1
        self.health = 4
        self.max_health = 4
        self.image11, self.rect = load_image('player11.bmp', -1)
        self.image11.convert()
        self.image12, self.rect = load_image('player12.bmp', -1)
        self.image12.convert()
        
        self.image31, self.rect = load_image('player31.bmp', -1)
        self.image31.convert()
        self.image32, self.rect = load_image('player32.bmp', -1)
        self.image32.convert()
                
        self.image51, self.rect = load_image('player51.bmp', -1)
        self.image51.convert()
        self.image52, self.rect = load_image('player52.bmp', -1)
        self.image52.convert()
        
        self.image71, self.rect = load_image('player71.bmp', -1)
        self.image71.convert()
        self.image72, self.rect = load_image('player72.bmp', -1)
        self.image72.convert()
        

        self.rect.center = (320,450)
        self.x_velocity = 0
        self.y_velocity = 0
        self.image = self.image11

    def update(self):
        global blocked_tile_list
        global mapx
        global mapy
        #move horiz
        self.rect.move_ip((self.x_velocity, 0))
        if self.rect.collidelist(blocked_tile_list) != -1:
            self.rect.move_ip((-self.x_velocity,0))

        #move vert
        self.rect.move_ip((0, self.y_velocity))
        if self.rect.collidelist(blocked_tile_list) != -1:
            self.rect.move_ip((0, -self.y_velocity))

        #swich maps    
        if self.rect.left < 0:
            self.rect.right = 598
            mapx -= 1
            blocked_tile_list = load_screen()
        elif self.rect.right > 600:
            self.rect.left = 2
            mapx += 1
            blocked_tile_list = load_screen()
            
        if self.rect.top < 0:
            self.rect.bottom = 598
            mapy -= 1
            blocked_tile_list = load_screen()
        elif self.rect.bottom >= 600:
            self.rect.top = 2
            mapy += 1
            blocked_tile_list = load_screen()
            
        #select current image
        if self.x_velocity == 0  and self.y_velocity < 0:
            self.direction = 1
        if self.x_velocity > 0 and self.y_velocity == 0:
            self.direction = 3
        if self.x_velocity == 0 and self.y_velocity > 0:
            self.direction = 5
        if self.x_velocity < 0 and self.y_velocity == 0:
            self.direction = 7
        if self.x_velocity == 0 and self.y_velocity == 0:
            #self.direction = 0
            pass

        if self.direction == 1:
            if self.flag % 6 == 3:
                self.image = self.image11
            elif self.flag % 6 ==0:
                self.image = self.image12
        elif self.direction == 3:
            if self.flag % 6 == 3:
                self.image = self.image31
            elif self.flag % 6 ==0:
                self.image = self.image32
        elif self.direction == 5:
            if self.flag % 6 == 3:
                self.image = self.image51
            elif self.flag % 6 ==0:
                self.image = self.image52
        elif self.direction == 7:
            if self.flag % 6 == 3:
                self.image = self.image71
            elif self.flag % 6 ==0:
                self.image = self.image72
                
        if self.x_velocity != 0 or self.y_velocity != 0:
            self.flag = self.flag + 1

class Boss1(pygame.sprite.Sprite):
    """This class is for the players character"""
    def __init__(self):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        #screen = pygame.display.get_surface()
        self.flag = 1
        self.hp = 120
        self.damage = 1
        self.image1, self.rect = load_image('boss11.bmp', -1)
        self.image1.convert()
        self.image2, self.rect = load_image('boss12.bmp', -1)
        self.image2.convert()
        

        self.rect.center = (300,100)
        self.distance = 0
        self.velocity = 1
        self.x_velocity = 0
        self.y_velocity = 0    
        self.image = self.image1

    def update(self):
        if self.hp <=0:
            effect_sprites.add(Stars(self.rect.center))
            self.kill()
            enemy_sprites.empty()
            enemy_weapon_sprites.empty()
            save_screen()
            return

        if self.distance == 0:
            #random distance from 3 to 15 turns
            self.distance=random.randint(3,15)
            #random x,y velocity form -2 to 2
            self.x_velocity=random.randint(-1,1) * self.velocity
            self.y_velocity=random.randint(-1,1) * self.velocity       



        #move horiz
        self.rect.move_ip((self.x_velocity, 0))
        buddy = False
        self.remove(enemy_sprites)
        if pygame.sprite.spritecollide(self, enemy_sprites,0):
                buddy = True
        if self.rect.collidelist(blocked_tile_list) != -1 or buddy:
            self.rect.move_ip((-self.x_velocity,0))
        self.add(enemy_sprites)

        if self.rect.left < 250:
            self.rect.left = 250
        elif self.rect.right > 350:
            self.rect.right = 350

        
        #move vert
        self.rect.move_ip((0, self.y_velocity))
        buddy = False
        self.remove(enemy_sprites)
        if pygame.sprite.spritecollide(self, enemy_sprites,0):
                buddy = True
        if self.rect.collidelist(blocked_tile_list) != -1 or buddy:
            self.rect.move_ip((0, -self.y_velocity))
        self.add(enemy_sprites)

        if self.rect.top <= 0:
            self.rect.top = 0
        elif self.rect.bottom >= 130:
            self.rect.bottom = 130

        self.distance -= 1

        if self.flag % 8 == 4:
            self.image = self.image1
        elif self.flag % 8 ==0:
            self.image = self.image2
            
        if self.x_velocity != 0 or self.y_velocity != 0:
            self.flag = self.flag + 1
            
        if self.flag % 200 == 0:
            #stars
            effect_sprites.add(Stars((100,300)))
            effect_sprites.add(Stars((500,300)))
        elif self.flag % 200 == 12:
            #enemy
            enemy_sprites.add(Enemy("wizard", (100,300), 2, 2, 1,"spell"))
            enemy_sprites.add(Enemy("wizard", (500,300), 2, 2, 1,"spell"))
            


class Boss2(pygame.sprite.Sprite):
    """This class is for the players character"""
    def __init__(self):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        #screen = pygame.display.get_surface()
        self.flag = 1
        self.hp = 40
        self.damage = 1
        self.image1, self.rect = load_image('boss21.bmp', -1)
        self.image1.convert()
        self.image2, self.rect = load_image('boss22.bmp', -1)
        self.image2.convert()
        self.projectile_image = 'fireball'
        

        self.rect.center = (400,300)
        self.distance = 0
        self.velocity = 1
        self.x_velocity = 0
        self.y_velocity = 0    
        self.image = self.image1

    def update(self):
        global player
        if self.hp <=0:
            effect_sprites.add(Stars(self.rect.center))
            self.kill()
            enemy_sprites.empty()
            save_screen()
            return

        if self.distance == 0:
            #random distance from 3 to 15 turns
            self.distance=random.randint(8,25)
            #random x,y velocity form -2 to 2
            self.x_velocity=random.randint(-1,1) * self.velocity
            self.y_velocity=random.randint(-1,1) * self.velocity       



        #move horiz
        self.rect.move_ip((self.x_velocity, 0))
        buddy = False
        #self.remove(enemy_sprites)
        #if pygame.sprite.spritecollide(self, enemy_sprites,0):
        #        buddy = True
        if self.rect.collidelist(blocked_tile_list) != -1 or buddy:
            self.rect.move_ip((-self.x_velocity,0))
        #self.add(enemy_sprites)

        if self.rect.left < 250:
            self.rect.left = 250
        

        
        #move vert
        self.rect.move_ip((0, self.y_velocity))
        buddy = False
        #self.remove(enemy_sprites)
        #if pygame.sprite.spritecollide(self, enemy_sprites,0):
        #        buddy = True
        if self.rect.collidelist(blocked_tile_list) != -1 or buddy:
            self.rect.move_ip((0, -self.y_velocity))
        #self.add(enemy_sprites)


        self.distance -= 1

        if self.flag % 8 == 4:
            self.image = self.image1
        elif self.flag % 8 ==0:
            self.image = self.image2
            
        if self.x_velocity != 0 or self.y_velocity != 0:
            self.flag = self.flag + 1

        #keep the player away so that he cant use his sword
        if player.rect.right > 200:
            player.rect.right = 200

            
        #shoot
        if random.randint(1,6) == 1:
            x_vel = - 4
            y_vel = random.randint(-4,4)
            enemy_weapon_sprites.add(Enemy_Projectile(self.projectile_image, self.rect.center, x_vel, y_vel, self.damage))



class Enemy(pygame.sprite.Sprite):
    """This class is for the players character"""
    def __init__(self, image, pos, vel, hp, damage=0, projectile_image=''):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        screen = pygame.display.get_surface()
        self.flag = 1
        self.hp = hp
        self.velocity = vel
        self.direction = 5
        self.projectile_image = projectile_image
        self.damage = damage
        self.image11, self.rect = load_image(image+'11.bmp', -1)
        self.image11.convert()
        self.image12, self.rect = load_image(image+'12.bmp', -1)
        self.image12.convert()
        
        self.image31, self.rect = load_image(image+'31.bmp', -1)
        self.image31.convert()
        self.image32, self.rect = load_image(image+'32.bmp', -1)
        self.image32.convert()
                
        self.image51, self.rect = load_image(image+'51.bmp', -1)
        self.image51.convert()
        self.image52, self.rect = load_image(image+'52.bmp', -1)
        self.image52.convert()
        
        self.image71, self.rect = load_image(image+'71.bmp', -1)
        self.image71.convert()
        self.image72, self.rect = load_image(image+'72.bmp', -1)
        self.image72.convert()
        

        self.rect.center = pos
        self.x_velocity = 0
        self.y_velocity = 0
            

        self.image = self.image51

    def update(self):
        if self.hp <=0:
            effect_sprites.add(Stars(self.rect.center))
            #drop a random object
            foo = random.randint(1, 5)
            if foo == 1 or foo == 2:
                object_sprites.add(Object("coin",self.rect.center, "label = font.render(str(money), 1, (0, 0, 0));label_pos = label.get_rect();label_pos.topleft = (705,50);screen.blit(label,label_pos);money+=1; label = font.render(str(money), 1, (200, 200, 200));label_pos.topleft = (705,50);screen.blit(label,label_pos)"))
            elif foo == 3:
                object_sprites.add(Object("health",self.rect.center, "\nif player.health < player.max_health:\n\ttext = font.render(str(player.health), 1, (0, 0, 0));screen.blit(text, textpos);player.health+=1;text = font.render(str(player.health), 1, (200, 200, 200));screen.blit(text, textpos)"))                
            self.kill()
            return

        x, y = player.rect.center
        if self.rect.left > x: #left
            self.x_velocity = -self.velocity
        elif self.rect.right < x:
            self.x_velocity = self.velocity
        else:
            self.x_velocity = 0
        if self.rect.top > y: #top?
            self.y_velocity = -self.velocity
        elif self.rect.bottom < y:
            self.y_velocity = self.velocity
        else:
            self.y_velocity = 0

        #move horiz
        self.rect.move_ip((self.x_velocity, 0))
        buddy = False
        self.remove(enemy_sprites)
        if pygame.sprite.spritecollide(self, enemy_sprites,0):
                buddy = True
        if self.rect.collidelist(blocked_tile_list) != -1 or buddy:
            self.rect.move_ip((-self.x_velocity,0))
        self.add(enemy_sprites)
        

        #move vert
        self.rect.move_ip((0, self.y_velocity))
        buddy = False
        self.remove(enemy_sprites)
        if pygame.sprite.spritecollide(self, enemy_sprites,0):
                buddy = True
        if self.rect.collidelist(blocked_tile_list) != -1 or buddy:
            self.rect.move_ip((0, -self.y_velocity))
        self.add(enemy_sprites)


        #select current image
        if self.x_velocity == 0  and self.y_velocity < 0:
            self.direction = 1
        elif self.x_velocity > 0 and self.y_velocity == 0:
            self.direction = 3
        elif self.x_velocity == 0 and self.y_velocity > 0:
            self.direction = 5
        elif self.x_velocity < 0 and self.y_velocity == 0:
            self.direction = 7
        elif self.x_velocity == 0 and self.y_velocity == 0:
            self.direction = 0

        if self.direction == 1:
            if self.flag % 6 == 3:
                self.image = self.image11
            elif self.flag % 6 ==0:
                self.image = self.image12
        elif self.direction == 3:
            if self.flag % 6 == 3:
                self.image = self.image31
            elif self.flag % 6 ==0:
                self.image = self.image32
        elif self.direction == 5:
            if self.flag % 6 == 3:
                self.image = self.image51
            elif self.flag % 6 ==0:
                self.image = self.image52
        elif self.direction == 7:
            if self.flag % 6 == 3:
                self.image = self.image71
            elif self.flag % 6 ==0:
                self.image = self.image72
                
        if self.x_velocity != 0 or self.y_velocity != 0:
            self.flag = self.flag + 1
        if self.projectile_image:
            if random.randint(1,60) == 1:
                dir = random.randint(1,8)
                if dir == 1:
                    x_vel = 0
                    y_vel = -2
                elif dir == 2:
                    x_vel = 2
                    y_vel = -2
                elif dir == 3:
                    x_vel = 2
                    y_vel = 0
                elif dir == 4:
                    x_vel = 2
                    y_vel = 2
                elif dir == 5:
                    x_vel = 0
                    y_vel = 2
                elif dir == 6:
                    x_vel = -2
                    y_vel = 2
                elif dir == 7:
                    x_vel = -2
                    y_vel = 0
                elif dir == 8:
                    x_vel = -2
                    y_vel = -2
                enemy_weapon_sprites.add(Enemy_Projectile(self.projectile_image, self.rect.center, x_vel, y_vel, self.damage))

            
class Object(pygame.sprite.Sprite):
    """This class is for the players character"""
    def __init__(self,image, pos, com):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        #screen = pygame.display.get_surface()
        self.image, self.rect = load_image(image+'.bmp', -1)
        self.rect.center = pos
        self.command = com

    def update(self):
        pass

class Enemy_Projectile(pygame.sprite.Sprite):
    """This class is for projectiles"""
    def __init__(self, image, pos, xv, yv,damage):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        #screen = pygame.display.get_surface()
        #self.image = image
        #self.rect = rect
        self.image, self.rect = load_image(image+'.bmp', -1)
        self.image.convert()
        self.rect.center = pos
        self.x_velocity = xv
        self.y_velocity = yv
        self.damage = damage
        self.hp = 1
            

    def update(self):
        self.rect.move_ip((self.x_velocity, self.y_velocity))
        if self.rect.centerx > 600:
            self.kill()
        elif self.rect.centerx < 0:
            self.kill()

        if self.rect.centery > 600:
            self.kill()
        elif self.rect.centery < 0:
            self.kill()
        if self.hp <=0:
            self.kill()


        
class Sword(pygame.sprite.Sprite):
    """This class is for projectiles"""
    def __init__(self, pos, dir, image, damage):
        pygame.sprite.Sprite.__init__(self) #call Sprite intializer
        self.image, self.rect = load_image(image+str(dir)+'.bmp', -1)
        self.image.convert()
        self.rect.center = pos
        self.flag = 0
        self.x_ref = player.rect.centerx
        self.y_ref = player.rect.centery
        self.damage = damage
        self.hp = 0            

    def update(self):
        self.flag += 1
        self.rect.move_ip((player.rect.centerx - self.x_ref,player.rect.centery- self.y_ref))
        self.x_ref = player.rect.centerx
        self.y_ref = player.rect.centery


        
        if self.flag == 9:
            self.kill()
        



def main():
    """this function is called when the program starts.
       it initializes everything it needs, then runs in
       a loop until the function returns."""
    random.seed()
    pygame.init()
    global screen
    global background

    #get the Players name
    global user_name
    if len(sys.argv) >= 2:
        user_name = sys.argv[1]
    else:
        user_name = 'default'
        

    global blocked_tile_list
    global mapx
    global mapy
    global s_weapon_image, s_weapon_damage, s_weapon_sound, s_weapon_sound_name
    s_weapon_image = 'null'
    s_weapon_damage = 0
    s_weapon_sound = load_sound('')
    s_weapon_sound_name = ''

    global a_weapon_image, a_weapon_damage, a_weapon_sound, a_weapon_sound_name
    a_weapon_image = 'null'
    a_weapon_damage = 0
    a_weapon_sound = load_sound('')
    a_weapon_sound_name = ''
    
    global money
    money = 0
    
    screen = pygame.display.set_mode((800, 600))#, FULLSCREEN)
    #pygame.mouse.set_visible(False)
    pygame.display.set_caption('The Adventure of Einar')


    #create the sprite groups
    global player 
    player = Player()
    global player_sprite
    player_sprite = pygame.sprite.RenderClear(player)

    global object_sprites
    global effect_sprites
    global enemy_sprites
    global enemy_weapon_sprites
    global player_weapon_sprites
    object_sprites = pygame.sprite.RenderClear()
    effect_sprites = pygame.sprite.RenderClear()
    enemy_sprites = pygame.sprite.RenderClear()
    enemy_weapon_sprites = pygame.sprite.RenderClear()
    player_weapon_sprites = pygame.sprite.RenderClear()


    #load font
    global font, textpos
    font = pygame.font.Font(None, 36)
    text = font.render(str(player.health), 1, (200, 200, 200))
    textpos = text.get_rect()
    textpos.topleft = (675,20)

    #load star images
    global star_images, star_rects
    star_image1, rect1 = load_image("star1.bmp", -1)
    star_image2, rect2 = load_image("star2.bmp", -1)
    star_image3, rect3 = load_image("star3.bmp", -1)
    star_images = (star_image1, star_image2, star_image3)
    star_rects = (rect1, rect2, rect3)

    #load explode sound
    explode = load_sound("hithard.wav")
    


    #load saved game info (if any)
    full_name = os.path.join('saved', user_name+".sav")
    if os.path.exists(full_name):
        load_game()

    #load the first map and create the background
    background = pygame.Surface(screen.get_size())
    background = background.convert()
    mapx = 500
    mapy = 500
    blocked_tile_list = load_screen()
    

    #Load music
    global shot1
    shot1 = load_sound("silent.wav")



    #### Enter the game loop ####
    
    running = 1
    while running:
        pygame.time.delay(10)
        
        for event in pygame.event.get():
            if event.type == QUIT:
                running = 0
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    running = 0
                elif event.key == K_LEFT:
                    player.x_velocity = -4
                    player.direction = 7
                elif event.key == K_RIGHT:
                    player.x_velocity = 4
                    player.direction = 3
                elif event.key == K_UP:
                    player.y_velocity = -4
                    player.direction = 1
                elif event.key == K_DOWN:
                    player.y_velocity = 4
                    player.direction = 5
                elif event.key == K_F1:
                    save_game()
                    print "game saved"
                elif event.key == K_F4:
                    clear_hp_text()
                    player.health = player.max_health
                    update_hp_text()
                elif event.key == K_F7:
                    screen = pygame.display.set_mode((800, 600))#, FULLSCREEN)
                    screen.blit(background, (0,0))
                    pygame.display.flip()
                    pygame.mouse.set_visible(True)
                    print_all_text()
                elif event.key == K_F8:
                    screen = pygame.display.set_mode((800, 600), FULLSCREEN)
                    screen.blit(background, (0,0))
                    pygame.display.flip()
                    print_all_text()
                    pygame.mouse.set_visible(False)

                elif event.key == K_s:
                    pos = player.rect.centerx, player.rect.top
                    dir = 1
                    if player.x_velocity == 0  and player.y_velocity < 0:
                        dir = 1
                        pos = player.rect.centerx, player.rect.top -15
                    elif player.x_velocity > 0 and player.y_velocity < 0:
                        dir = 2
                        pos = player.rect.right+10, player.rect.top -10
                    elif player.x_velocity > 0 and player.y_velocity == 0:
                        dir = 3
                        pos = player.rect.right+10, player.rect.centery
                    elif player.x_velocity > 0 and player.y_velocity > 0:
                        dir = 4
                        pos = player.rect.right+10, player.rect.bottom+10
                    elif player.x_velocity == 0 and player.y_velocity > 0:
                        dir = 5
                        pos = player.rect.centerx, player.rect.bottom+15
                    elif player.x_velocity < 0 and player.y_velocity > 0:
                        dir = 6
                        pos = player.rect.left-10, player.rect.bottom+10
                    elif player.x_velocity < 0 and player.y_velocity == 0:
                        dir = 7
                        pos = player.rect.left-10, player.rect.centery
                    elif player.x_velocity < 0 and player.y_velocity < 0:
                        dir = 8
                        pos = player.rect.left-10, player.rect.top -10
                    elif player.x_velocity == 0 and player.y_velocity == 0:
                        dir = player.direction
                        if dir == 1:
                            pos = player.rect.centerx, player.rect.top - 14
                        elif dir == 3:
                            pos = player.rect.right + 10, player.rect.centery
                        elif dir == 5:
                            pos = player.rect.centerx, player.rect.bottom + 14
                        elif dir == 7:
                            pos = player.rect.left - 10, player.rect.centery
                    
                    player_weapon_sprites.add(Sword(pos,dir,s_weapon_image,s_weapon_damage))
                    s_weapon_sound.play()

                elif event.key == K_a and player.health >= player.max_health and a_weapon_sound_name != '':
                    dir = 1
                    if player.x_velocity == 0  and player.y_velocity < 0:
                        #dir = 1
                        x_vel = 0
                        y_vel = -6
                    elif player.x_velocity > 0 and player.y_velocity < 0:
                        #dir = 2
                        x_vel = 6
                        y_vel = -6
                    elif player.x_velocity > 0 and player.y_velocity == 0:
                        #dir = 3
                        x_vel = 6
                        y_vel = 0
                    elif player.x_velocity > 0 and player.y_velocity > 0:
                        #dir = 4
                        x_vel = 6
                        y_vel = 6
                    elif player.x_velocity == 0 and player.y_velocity > 0:
                        #dir = 5
                        x_vel = 0
                        y_vel = 6
                    elif player.x_velocity < 0 and player.y_velocity > 0:
                        #dir = 6
                        x_vel = -6
                        y_vel = 6
                    elif player.x_velocity < 0 and player.y_velocity == 0:
                        #dir = 7
                        x_vel = -6
                        y_vel = 0
                    elif player.x_velocity < 0 and player.y_velocity < 0:
                        #dir = 8
                        x_vel = -6
                        y_vel = -6
                    elif player.x_velocity == 0 and player.y_velocity == 0:
                        dir = player.direction
                        if dir == 1:
                            x_vel = 0
                            y_vel = -6
                        elif dir == 3:
                            x_vel = 6
                            y_vel = 0
                        elif dir == 5:
                            x_vel = 0
                            y_vel = 6
                        elif dir == 7:
                            x_vel = -6
                            y_vel = 0

                    player_weapon_sprites.add(Enemy_Projectile(a_weapon_image, player.rect.center, x_vel, y_vel, a_weapon_damage))
                    a_weapon_sound.play()

                    
            elif event.type == KEYUP:
                if event.key == K_LEFT:
                    player.x_velocity = 0
                elif event.key == K_RIGHT:
                    player.x_velocity = 0
                elif event.key == K_UP:
                    player.y_velocity = 0
                elif event.key == K_DOWN:
                    player.y_velocity = 0

        #Update all sprites
        player_sprite.update()
        player_weapon_sprites.update()
        object_sprites.update()
        effect_sprites.update()
        enemy_sprites.update()
        enemy_weapon_sprites.update()

                
        #see if the player touched an object
        for hit in pygame.sprite.groupcollide(object_sprites, player_sprite, 1, 0).keys():
            exec("global s_weapon_image, s_weapon_damage, s_weapon_sound, s_weapon_sound_name,a_weapon_image, a_weapon_damage, a_weapon_sound, a_weapon_sound_name, money, player;"+hit.command)
            save_game()

        #see if enemy weapons hit the player
        dic = pygame.sprite.groupcollide(enemy_weapon_sprites, player_sprite, 1, 0)
        for hit in dic.keys():
            text = font.render(str(player.health), 1, (0, 0, 0))
            screen.blit(text, textpos)
            player.health -= hit.damage
            text = font.render(str(player.health), 1, (200, 200, 200))
            screen.blit(text, textpos)
            xv = 0
            yv = 0
            if player.rect.centerx > hit.rect.centerx:
                xv = 2
            elif player.rect.centerx < hit.rect.centerx:
                xv = -2
            if player.rect.centery > hit.rect.centery:
                yv = 2
            elif player.rect.centery < hit.rect.centery:
                yv = -2
            for foo in range(20):
                #move horiz
                player.rect.move_ip((xv, 0))
                if player.rect.collidelist(blocked_tile_list) != -1:
                    player.rect.move_ip((-xv,0))

                if player.rect.left < 0:
                    player.rect.right = 598
                    mapx -= 1
                    blocked_tile_list = load_screen()
                elif player.rect.right > 600:
                    player.rect.left = 2
                    mapx += 1
                    blocked_tile_list = load_screen()


                #move vert
                player.rect.move_ip((0, yv))
                if player.rect.collidelist(blocked_tile_list) != -1:
                    player.rect.move_ip((0, -yv))

                if player.rect.top < 0:
                    player.rect.bottom = 598
                    mapy -= 1
                    blocked_tile_list = load_screen()
                elif player.rect.bottom >= 600:
                    player.rect.top = 2
                    mapy += 1
                    blocked_tile_list = load_screen()


        #see if enemy weapons hit the player's weapons
        dic = pygame.sprite.groupcollide(enemy_weapon_sprites, player_weapon_sprites, 1, 0)
        for hit in dic.keys():
            explode.play()           


        #see if enemy hit player
        dic = pygame.sprite.groupcollide(enemy_sprites, player_sprite, 0, 0)
        for hit in dic.keys():
            text = font.render(str(player.health), 1, (0, 0, 0))
            screen.blit(text, textpos)
            player.health -= hit.damage
            text = font.render(str(player.health), 1, (200, 200, 200))
            screen.blit(text, textpos)
            xv = 0
            yv = 0
            if player.rect.centerx > hit.rect.centerx:
                xv = 2
            elif player.rect.centerx < hit.rect.centerx:
                xv = -2
            if player.rect.centery > hit.rect.centery:
                yv = 2
            elif player.rect.centery < hit.rect.centery:
                yv = -2
            for foo in range(20):
                #move horiz
                player.rect.move_ip((xv, 0))
                if player.rect.collidelist(blocked_tile_list) != -1:
                    player.rect.move_ip((-xv,0))

                if player.rect.left < 0:
                    player.rect.right = 598
                    mapx -= 1
                    blocked_tile_list = load_screen()
                elif player.rect.right > 600:
                    player.rect.left = 2
                    mapx += 1
                    blocked_tile_list = load_screen()


                #move vert
                player.rect.move_ip((0, yv))
                if player.rect.collidelist(blocked_tile_list) != -1:
                    player.rect.move_ip((0, -yv))

                if player.rect.top < 0:
                    player.rect.bottom = 598
                    mapy -= 1
                    blocked_tile_list = load_screen()
                elif player.rect.bottom >= 600:
                    player.rect.top = 2
                    mapy += 1
                    blocked_tile_list = load_screen()




            
            if player.health <= 0:
                running = 0

        #see if the player's weapons hit enemy
        dic = pygame.sprite.groupcollide(enemy_sprites, player_weapon_sprites, 0, 0)
        for enemy in dic.keys():
            explode.play()
            for weapon in dic[enemy]:
                enemy.hp -= weapon.damage
                xv = 0
                yv = 0
                weapon.hp -= 1
                if enemy.rect.centerx > player.rect.centerx:
                    xv = 2
                elif enemy.rect.centerx < player.rect.centerx:
                    xv = -2
                if player.rect.centery > player.rect.centery:
                    yv = 2
                elif enemy.rect.centery < player.rect.centery:
                    yv = -2
                for foo in range(20):
                    #move horiz
                    enemy.rect.move_ip((xv, 0))
                    if enemy.rect.collidelist(blocked_tile_list) != -1:
                        enemy.rect.move_ip((-xv,0))
                    #move vert
                    enemy.rect.move_ip((0, yv))
                    if enemy.rect.collidelist(blocked_tile_list) != -1:
                        enemy.rect.move_ip((0, -yv))


                
                
        
        #Clear Everything
        enemy_sprites.clear(screen, background)
        object_sprites.clear(screen, background)
        effect_sprites.clear(screen, background)
        player_sprite.clear(screen, background)
        enemy_weapon_sprites.clear(screen, background)
        player_weapon_sprites.clear(screen, background)
        player_weapon_sprites.update()
        
            
        #Draw Everything
        #etorpeado_sprites.draw(screen)
        object_sprites.draw(screen)
        effect_sprites.draw(screen)
        enemy_sprites.draw(screen)
        enemy_weapon_sprites.draw(screen)
        player_sprite.draw(screen)
        player_weapon_sprites.draw(screen)



        
        pygame.display.flip()

    #Game is finnished wait 3 seconds before exiting
    pygame.time.delay(2000)
    screen = pygame.display.set_mode((800, 600))    

#this calls the 'main' function when this script is executed
if __name__ == '__main__': main()
